unit GradBtn;
{This component is freeware.  Use, abuse it, modify it, improve it, take out
 parts you don't need/want.  You don't need to credit the author if you use
 this in an application.  However, if you must communicate with me I'm at
 harmdog@ne.uswest.net }

{Version 1.0 11/97 Initial release to the Public
 Version 1.1 12/97 Added Presets.  Add KeyPress to allow Enter key to act as
              Click. Also added btnFrameSize property, but I don't like it much
             'cause it overlays the gradient, and I'm too lazy to figure
             out a solution.}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

{I learned the basic gradient technique from a component by John Baumbach.
 I added several more types of gradient and 3D text.  His component was
 derived from a SpeedButton.  I found his component at the Delphi Deli,
 (www.delphideli.com), a great place on the Web.  There are a lot of
 comments in here, which kind of trashes up the code, but I wanted to
 make this understandable to just about anybody, so have tried to explain
 everything that's going on.}

{These are the six different gradient styles available.}
type
  TGradientStyle = (gsHorizontal, gsVertical, gsElliptic, gsRectangle,
                    gsVertCenter, gsHorizCenter);

{Text can be normal, or 3D raised/inset}
type
  TTextStyle = (tsNormal, tsRaised, tsInset);

{ShadeType refers to the 3D text effect.}
type
  TShadeType = (stLight, stHeavy);

{Added some presets to make things easier}
type TPreset = (None, GoldCoast, LimeBox, Aluminum, SunRise);

type
  TGradBtn = class(TCustomControl)
  private
    { Private declarations }
    FBtnHiliteClr : TColor;   //Hilite around button
    FBtnShadowClr : TColor;   //Shadow around button
    FBtnFrameSize : Integer;  //Size of 3D Frame
    FTxtHiliteClr : TColor;  //Text Hilite
    FTxtShadowClr : TColor;  //Text Shadow
    FBeginClr : TColor;    //Start color for gradient
    FEndClr : TColor;     //End color for gradient
    FSwapClr : Boolean;    //Swap start-end colors on button press?
    FGradientStyle : TGradientStyle;  //One of six choices
    FTextStyle : TTextStyle;  //Raised, Inset, or None
    FShadeType : TShadeType; //Light or heavy text shading
    mDn, mUp : Boolean;    //Flags for Mouse Up and Down
    bm : TBitmap;          //WorkHorse internal bmp
    TmpClr : TColor;       //Holding area for Color during Mouse Down.
    FPreset : TPreset;   //Variable for presets
    procedure Paint; override;  //Override Paint - entire control is painted.
    procedure CMTextChanged(var Msg:TMessage); message CM_TEXTCHANGED;
    //I had to add this to get the Caption to change while designing
    procedure WMEraseBkgnd(Var Msg : TMessage); message WM_ERASEBKGND;
    //I added this to help reduce flicker and speed things up.
    procedure MouseDown(Button:TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button:TMouseButton; Shift:TShiftState;
      X, Y: Integer); override;
    //Mouse up and down are what I use to swap gradient colors
    procedure SetBtnHilite(Value: TColor);  //Set color of top and left of frame
    procedure SetBtnShadow(Value: TColor);  //Set Color of right and bottom of frame
    procedure SetBtnFrameSize(Value: Integer); //Set size of 3D frame
    procedure SetTxtHilite(Value: TColor);  //Set color for text 3D effect hilite
    procedure SetTxtShadow(Value: TColor);  //Set color for text 3D effect sadow
    procedure SetBeginClr(Value : TColor);  //Set start color of gradient
    procedure SetEndClr(Value : TColor);    //Set end color of gradient
    procedure SetSwap(Value : Boolean);  //Swap the start-end true-false
    procedure SetStyle(Value: TTextStyle);  //Set Caption 3D style
    procedure SetShade(Value: TShadeType);  //Set Caption shading type
    procedure SetPreset(Value: TPreset);   //Set preset values
    procedure DoRaise(offset: Integer);   //This draws the 3D Caption 'raised'
    procedure DoInset(offset: Integer);  //This draws te 3D caption 'inset'
    procedure DoNorm(offset: Integer);   //This draws the caption with no 3D
    procedure SetGradient(Value : TGradientStyle);  //Choose gradient fill style
    procedure DoHorizontal(fr, fg, fb, dr, dg, db : Integer); //These all draw
    procedure DoVertical(fr, fg, fb, dr, dg, db : Integer); // the gradients.
    procedure DoElliptic(fr, fg, fb, dr, dg, db : Integer); // The fr fg fb etc
    procedure DoRectangle(fr, fg, fb, dr, dg, db : Integer); // are color values.
    procedure DoVertCenter(fr, fg, fb, dr, dg, db : Integer);
    procedure DoHorizCenter(fr, fg, fb, dr, dg, db : Integer);
  protected
    { Protected declarations }
    {I added the following to allow the Enter key to behave as a Click.}
    procedure KeyPress(var Key: Char); override;
  public
    { Public declarations }
    {Almost all components override the constructor, so that default values for
     new properties can be set.  The destructor is usually overridden so cleanup
     can be done - like releasing resources so they don't cause memory leaks.}
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    {This first batch of stuff is all the new properties I added for this
     component.  As long as the properties are part of the usual types, you
     don't have to define your own property editor. I love Delphi.}
    property BtnHiliteClr : TColor read FBtnHiliteClr
       write SetBtnHilite default clBtnHighLight;
    property BtnShadowClr : TColor read FBtnShadowClr
       write SetBtnShadow default clBtnShadow;
    property BtnFrameSize : Integer read FBtnFrameSize
       write SetBtnFrameSize default 1;
    property TxtHiliteClr : TColor read FTxtHiliteClr
       write SetTxtHilite default clBtnHighLight;
    property TxtShadowClr : TColor read FTxtShadowClr
       write SetTxtShadow default clBtnShadow;
    property BeginColor : TColor read FBeginClr
       write SetBeginClr default clNavy;
    property EndColor : TColor read FEndClr
       write SetEndClr default clAqua;
    property SwapColors : Boolean read FSwapClr
       write SetSwap default TRUE;
    property GradientStyle : TGradientStyle read FGradientStyle
       write SetGradient default gsVertCenter;
    property TextStyle : TTextStyle read FTextStyle
       write SetStyle default tsNormal;
    property TextShadeType : TShadeType read FShadeType
       write SetShade default stLight;
    property Presets : TPreset read FPreset
       write SetPreset default None;
    property Caption;      //Added these so they show up in the Obj Insp.
    property OnClick;      // When deriving from a Custom type control,
    property OnMouseDown;  // these don't normally show up unless you specifically
    property OnMouseUp;    // declare them yourself.  You can find them by
    property Visible;      // going through the browser.
    property Enabled;
    property Font;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseMove;
  end;

procedure Register;

implementation

constructor TGradBtn.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FBtnHiliteClr := clBtnHighLight;   //Default Hilite Color
  FBtnShadowClr := clBtnShadow;      //Default Shadow Color
  FBtnFrameSize := 1;                //Default frame size
  FTxtHiliteClr := clBtnHighLight;   //Default Text Hilite
  FTxtShadowClr := clBtnShadow;      //Default Text Shadow
  FBeginClr := clNavy;            //Start Color is at the Top of button
  FEndClr := clAqua;             //Stop Color is at the Bottom of button
  Height := 25;                   //Default button size
  Width := 75;
  bm := TBitmap.Create;          //Create the internal BMP
  bm.Canvas.Pen.Style:= psSolid; //These may already be the default,
  bm.Canvas.Pen.Mode:= pmCopy;   // but just in case...
  mDn := FALSE;                   //Mouse is UP to begin with
  mUp := TRUE;
  Font.Color := clBlack;         //Default Font Color
  ControlStyle := ControlStyle + [csOpaque];   //This reduces flicker
  FSwapClr := TRUE;               //By default, we swap colors on Mouse Down
  FShadeType := stLight;          //Light text shading by default
  FTextStyle := tsRaised;         //Raised '3d' text
  FGradientStyle := gsVertCenter;   //Top & Bottom to Center for gradient Fill default
  FPreset := None;
  TabStop := TRUE;      //Made this the default so button can have Focus
end;

{Everything pretty much happens in the Paint procedure; a lot of it is farmed
 out to other procedures just so the logic in the paint procedure can be
 understood a little better.  Here's the basic process for creating a gradient:
 Get the separate R, G, and B values of the BeginClr Color.  Then, find the
 difference between that and the R, G, and B values of the EndClr Color.  Using
 these two sets of values, draw a series of 'shapes' using a series of colors
 between the begin and end.  After the gradient is drawn, add a frame and the
 Caption.}
procedure TGradBtn.Paint;
var
  FromR, FromG, FromB : Integer; //These are the separate color values for RGB
  DiffR, DiffG, DiffB : Integer; // of color values.
  rct : TRect;                   //Rectangle used to draw frame around button
  offset : Integer;              //Used for Caption location during Mouse Down
{To speed things up and reduce flicker, I use a Bitmap to draw the button in
 its entirety, ten BitBlt it to the canvas of the control.}
begin
  inherited;                  // PROBABLY DON'T NEED THIS
  bm.Canvas.Font.Color := Font.Color; //Keep Bitmap in synch with settings.
  bm.Canvas.Font := Font;     // Set the Bitmap font to match control font
  bm.Width := Width;          //Set BMP dimensions to match control's
  bm.Height := Height;
  rct := Rect(0,0,Width,Height);  //Set rectangle size for later use
  FromR := FBeginClr and $000000ff;  //Strip out separate RGB values
  FromG := (FBeginClr shr 8) and $000000ff;
  FromB := (FBeginClr shr 16) and $000000ff;
  DiffR := (FEndClr and $000000ff) - FromR;   //Find the difference
  DiffG := ((FEndClr shr 8) and $000000ff) - FromG;
  DiffB := ((FEndClr shr 16) and $000000ff) - FromB;
  //Depending on gradient style selected, go draw it on the Bitmap canvas.
  if FGradientStyle = gsHorizontal then
    DoHorizontal(FromR, FromG, FromB, DiffR, DiffG, DiffB);
  if FGradientStyle = gsVertical then
    DoVertical(FromR, FromG, FromB, DiffR, DiffG, DiffB);
  if FGradientStyle = gsElliptic then
    DoElliptic(FromR, FromG, FromB, DiffR, DiffG, DiffB);
  if FGradientStyle = gsRectangle then
    DoRectangle(FromR, FromG, FromB, DiffR, DiffG, DiffB);
  if FGradientStyle = gsVertCenter then
    DoVertCenter(FromR, FromG, FromB, DiffR, DiffG, DiffB);
  if FGradientStyle = gsHorizCenter then
    DoHorizCenter(FromR, FromG, FromB, DiffR, DiffG, DiffB);
  //By setting the Brush style to Clear, it will draw without overlaying bkgrnd
  bm.Canvas.Brush.Style := bsClear;  //Gradient is done, time for Hilite-Shadow
  if mUp = TRUE then begin       //If Mouse up do normal button
    offset := 0;  //offset is used to move the Caption down a notch when the
                  // button is pressed.
    Frame3D(bm.Canvas,rct,FBtnHiliteClr,FBtnShadowClr,FBtnFrameSize); //Draw frame
  end
  else begin
    offset := 1; //If mouse down, we want caption to 'move' and reverse the frame
    Frame3D(bm.Canvas,rct,FBtnShadowClr,FBtnHiliteClr,FBtnFrameSize);
  end;
  if FTextStyle = tsRaised then DoRaise(offset);  //Based on style, we draw
  if FTextStyle = tsInset then DoInset(offset);   // the caption on the
  if FTextStyle = tsNormal then DoNorm(offset);   // button
  {Finally, the button is all painted on the bitmap canvas.  Now we just need
   to copy it to the canvas of our control.  BitBlt is one method; there are
   several others.}
  BitBlt(Canvas.Handle,0,0,Width,Height,bm.Canvas.Handle,0,0,SRCCOPY);
end;

procedure TGradBtn.SetBtnHilite(Value: TColor);
begin
  if FBtnHiliteClr <> Value then begin
    FBtnHiliteClr := Value;
    Invalidate;  //Invalidate tells windows to do a Paint.  Our control
  end;           // goes through its Paint procedure.
end;

procedure TGradBtn.SetBtnShadow(Value: TColor);
begin
  if FBtnShadowClr <> Value then begin
    FBtnShadowClr := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetBtnFrameSize(Value: Integer);
begin
  if FBtnFrameSize <> Value then begin
    if Value > ((Width Div 2) - 5) then exit;
    if Value > ((Height Div 2) - 5) then exit;
    FBtnFrameSize := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetTxtHilite(Value: TColor);
begin
  if FTxtHiliteClr <> Value then begin
    FTxtHiliteClr := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetTxtShadow(Value: TColor);
begin
  if FTxtShadowClr <> Value then begin
    FTxtShadowClr := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetBeginClr(Value : TColor);
begin
  if FBeginClr <> Value then begin
    FBeginClr := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetEndClr(Value : TColor);
begin
  if FEndClr <> Value then begin
    FEndClr := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetSwap(Value : Boolean);
begin
  if FSwapClr <> Value then
    FSwapClr := Value;
end;

procedure TGradBtn.MouseDown;
begin
  inherited;
  mDn := TRUE;
  mUp := FALSE;
  if FSwapClr = TRUE then begin    //If SwapClrs is true, we need to save
    TmpClr := FBeginClr;           // the original stuff.  Hence, TmpClr
    FBeginClr := FEndClr;
    FEndClr := TmpClr;
  end;
  Invalidate;
end;

procedure TGradBtn.MouseUp;
begin
  inherited;
  if FSwapClr = TRUE then begin
    TmpClr := FBeginClr;
    FBeginClr := FEndClr;
    FEndClr := TmpClr;
  end;
  mUp := TRUE;
  mDn := FALSE;
  Invalidate;
end;

procedure TGradBtn.CMTextChanged(var Msg:TMessage);
begin
  Inherited;
  Invalidate;
end;

procedure TGradBtn.WMEraseBkgnd(var Msg:TMessage);
begin
  Msg.Result := 0;  //We intercept this message and tell
end;                // Windows it processed OK.

procedure TGradBtn.SetGradient(Value : TGradientStyle);
begin
  if FGradientStyle <> Value then begin
    FGradientStyle := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetStyle(Value: TTextStyle);
begin
  {When user selects a style, set the variable to new style and do
   an invalidate to force a re-paint of the component.}
  if FTextStyle <> Value then begin
    FTextStyle := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetShade(Value: TShadeType);
begin
  {When user selects a shade, set the variable to new type and do
   an invalidate to force a re-paint of the component.}
  if FShadeType <> Value then begin
    FShadeType := Value;
    Invalidate;
  end;
end;

procedure TGradBtn.SetPreset(Value: TPreset);
begin
 if csDesigning in ComponentState then begin
  case Value of
    None : begin;
      FBeginClr := clNavy;
      FEndClr := clAqua;
      Font.Size := 8;
      Font.Name := 'MS Sans Serif';
      Font.Color := clBlack;
      Font.Style := [];
      FGradientStyle := gsVertCenter;
      FShadeType := stLight;
      FTextStyle := tsRaised;
      FTxtHiliteClr := clBtnHighLight;
      FTxtShadowClr := clBtnShadow;
      FBtnHiliteClr := clBtnHighlight;
      FBtnShadowClr := clBtnShadow;
    end;
    GoldCoast : begin
      FBeginClr := clYellow;
      FEndClr := clOlive;
      Font.Size := 10;
      Font.Name := 'Arial';
      Font.Color := $0000bbbb;
      Font.Style := [fsBold];
      FGradientStyle := gsVertical;
      FShadeType := stHeavy;
      FTextStyle := tsRaised;
      FTxtHiliteClr := $00bfffff;
      FTxtShadowClr := $00007171;
      FBtnHiliteClr := $00b3ffff;
      FBtnShadowClr := $00616161;
    end;
    LimeBox : begin
      FBeginClr := clBlack;
      FEndClr := clLime;
      Font.Size := 8;
      Font.Name := 'Courier';
      Font.Color := clGreen;
      Font.Style := [];
      FGradientStyle := gsRectangle;
      FShadeType := stLight;
      FTextStyle := tsRaised;
      FTxtHiliteClr := $00acffac;
      FTxtShadowClr := $00004000;
      FBtnHiliteClr := clBtnHighLight;
      FBtnShadowClr := clBtnShadow;
    end;
    Aluminum : begin
      FBeginClr := $00cecece;
      FEndClr := $005f5f5f;
      Font.Size := 10;
      Font.Name := 'Times New Roman';
      Font.Color := clBlack;
      Font.Style := [fsBold];
      FGradientStyle := gsVertical;
      FShadeType := stHeavy;
      FTextStyle := tsInset;
      FTxtHiliteClr := clBtnHighlight;
      FTxtShadowClr := clBtnShadow;
      FBtnHiliteClr := clWhite;
      FBtnShadowClr := clBlack;
    end;
    SunRise : begin
      FBeginClr := clYellow;
      FEndClr := clRed;
      Font.Size := 8;
      Font.Name := 'MS Sans Serif';
      Font.Color := $00000040;
      Font.Style := [fsBold];
      FGradientStyle := gsHorizontal;
      FShadeType := stHeavy;
      FTextStyle := tsRaised;
      FTxtHiliteClr := $00cfe7e6;
      FTxtShadowClr := $000058b0;
      FBtnHiliteClr := clBtnHighlight;
      FBtnShadowClr := clBtnShadow;
    end;
    else
      Exit;
  end;
  FPreset := Value;
  Invalidate;
 end;
end;

{I'll explain a little about the Horizontal gradient, the other styles are all
 consistent with their logic.  The six R, G, and B values are passed to us.
 We define some local variables we'll need: a rectangle, a FOR loop counter,
 and our own RGB numbers.  For a horizontal gradient, we'll draw a series of
 rectangles, each one a little closer in color to the EndClr value.  A horizontal
 gradient rectangle will always be from the top to the bottom of the canvas,
 so we set top to 0 and bottom to however tall our control is.  Then, we draw
 a series of 255 rectangles.  The starting point and width of each will depend
 on the actual width of our control.  It starts out on the left, draws the
 first rectangle in a color that's a percentage of the difference plus the
 starting color.  As I increments through the loop, the rectangles move to the
 right and the color gets closer and closer to the EndClr.}
procedure TGradBtn.DoHorizontal(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
begin
  ColorRect.Top:= 0;                //Set rectangle top
  ColorRect.Bottom := Height;
  for I := 0 to 255 do begin         //Make lines (rectangles) of color
    ColorRect.Left:= MulDiv (I, Width, 256);    //Find left for this color
    ColorRect.Right:= MulDiv (I + 1, Width, 256);   //Find Right
    R := fr + MulDiv(I, dr, 255);    //Find the RGB values
    G := fg + MulDiv(I, dg, 255);
    B := fb + MulDiv(I, db, 255);
    bm.Canvas.Brush.Color := RGB(R, G, B);   //Plug colors into brush
    bm.Canvas.FillRect(ColorRect);           //Draw on Bitmap
  end;
end;

procedure TGradBtn.DoVertical(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
begin
  ColorRect.Left:= 0;                //Set rectangle left&right
  ColorRect.Right:= Width;
  for I := 0 to 255 do begin         //Make lines (rectangles) of color
    ColorRect.Top:= MulDiv (I, Height, 256);    //Find top for this color
    ColorRect.Bottom:= MulDiv (I + 1, Height, 256);   //Find Bottom
    R := fr + MulDiv(I, dr, 255);    //Find the RGB values
    G := fg + MulDiv(I, dg, 255);
    B := fb + MulDiv(I, db, 255);
    bm.Canvas.Brush.Color := RGB(R, G, B);   //Plug colors into brush
    bm.Canvas.FillRect(ColorRect);           //Draw on Bitmap
  end;
end;

procedure TGradBtn.DoElliptic(fr, fg, fb, dr, dg, db : Integer);
var
  I: Integer;
  R, G, B : Byte;
  Pw, Ph : Real;
  x1,y1,x2,y2 : Real;
{The elliptic is a bit different, since I had to use real numbers. I cut down
 on the number (to 155 instead of 255) of iterations in an attempt to speed
 things up, to no avail.  I think it just takes longer for windows to draw an
 ellipse as opposed to a rectangle.}
begin
  bm.Canvas.Pen.Style := psClear;
  bm.Canvas.Pen.Mode := pmCopy;
  x1 := 0 - (bm.Width / 4);
  x2 := bm.Width + (bm.Width / 4);
  y1 := 0 - (bm.Height / 4);
  y2 := bm.Height + (bm.Height / 4);
  Pw := ((bm.Width / 4) + (bm.Width / 2)) / 155;
  Ph := ((bm.Height / 4) + (bm.Height / 2)) / 155;
  for I := 0 to 155 do begin         //Make ellipses of color
    x1 := x1 + Pw;
    x2 := X2 - Pw;
    y1 := y1 + Ph;
    y2 := y2 - Ph;
    R := fr + MulDiv(I, dr, 155);    //Find the RGB values
    G := fg + MulDiv(I, dg, 155);
    B := fb + MulDiv(I, db, 155);
    bm.Canvas.Brush.Color := R or (G shl 8) or (b shl 16);   //Plug colors into brush
    bm.Canvas.Ellipse(Trunc(x1),Trunc(y1),Trunc(x2),Trunc(y2));
  end;
  bm.Canvas.Pen.Style := psSolid;
end;

procedure TGradBtn.DoRectangle(fr, fg, fb, dr, dg, db : Integer);
var
  I: Integer;
  R, G, B : Byte;
  Pw, Ph : Real;
  x1,y1,x2,y2 : Real;
begin
  bm.Canvas.Pen.Style := psClear;
  bm.Canvas.Pen.Mode := pmCopy;
  x1 := 0;
  x2 := bm.Width;
  y1 := 0;
  y2 := bm.Height;
  Pw := (bm.Width / 2) / 255;
  Ph := (bm.Height / 2) / 255;
  for I := 0 to 255 do begin         //Make rectangles of color
    x1 := x1 + Pw;
    x2 := X2 - Pw;
    y1 := y1 + Ph;
    y2 := y2 - Ph;
    R := fr + MulDiv(I, dr, 255);    //Find the RGB values
    G := fg + MulDiv(I, dg, 255);
    B := fb + MulDiv(I, db, 255);
    bm.Canvas.Brush.Color := RGB(R, G, B);   //Plug colors into brush
    bm.Canvas.FillRect(Rect(Trunc(x1),Trunc(y1),Trunc(x2),Trunc(y2)));
  end;
  bm.Canvas.Pen.Style := psSolid;
end;

procedure TGradBtn.DoVertCenter(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
  Haf : Integer;
begin
  Haf := bm.Height Div 2;
  ColorRect.Left := 0;
  ColorRect.Right := bm.Width;
  for I := 0 to Haf do begin
    ColorRect.Top := MulDiv (I, Haf, Haf);
    ColorRect.Bottom := MulDiv (I + 1, Haf, Haf);
    R := fr + MulDiv(I, dr, Haf);
    G := fg + MulDiv(I, dg, Haf);
    B := fb + MulDiv(I, db, Haf);
    bm.Canvas.Brush.Color := RGB(R, G, B);
    bm.Canvas.FillRect(ColorRect);
    ColorRect.Top := bm.Height - (MulDiv (I, Haf, Haf));
    ColorRect.Bottom := bm.Height - (MulDiv (I + 1, Haf, Haf));
    bm.Canvas.FillRect(ColorRect);
  end;
end;

procedure TGradBtn.DoHorizCenter(fr, fg, fb, dr, dg, db : Integer);
var
  ColorRect: TRect;
  I: Integer;
  R, G, B : Byte;
  Haf : Integer;
begin
  Haf := bm.Width Div 2;
  ColorRect.Top := 0;
  ColorRect.Bottom := bm.Height;
  for I := 0 to Haf do begin
    ColorRect.Left := MulDiv (I, Haf, Haf);
    ColorRect.Right := MulDiv (I + 1, Haf, Haf);
    R := fr + MulDiv(I, dr, Haf);
    G := fg + MulDiv(I, dg, Haf);
    B := fb + MulDiv(I, db, Haf);
    bm.Canvas.Brush.Color := RGB(R, G, B);
    bm.Canvas.FillRect(ColorRect);
    ColorRect.Left := bm.Width - (MulDiv (I, Haf, Haf));
    ColorRect.Right := bm.Width - (MulDiv (I + 1, Haf, Haf));
    bm.Canvas.FillRect(ColorRect);
  end;
end;

procedure TGradBtn.DoRaise(offset: Integer);
{This draws 'raised' 3D text for the Caption.  Inset is the same principal,
 the colors are just reversed.  The brush style is set to Clear so the
 background is not erased.  The highlight colored text is drawn on the canvas
 at -1 up and left of center.  If heavy shading is set, the shadow colored
 text is drawn at +1 rigt and down of center.  Finally, the Font.Color colored
 text is drawn in exact center.  The offset parameter is used to 'move' the
 whole shebang down and right 1 if the Mouse is Down.}
begin
    bm.Canvas.Brush.Style := bsClear;
    bm.Canvas.Font.Color := FTxtHiliteClr;
    bm.Canvas.TextOut(((bm.Width-bm.Canvas.TextWidth(Caption)) div 2)-1+offset,
      ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)-1+offset,Caption);
    if FShadeType = stHeavy then begin
      bm.Canvas.Font.Color := FTxtShadowClr;
      bm.Canvas.TextOut(((bm.Width-bm.Canvas.TextWidth(Caption)) div 2)+1+offset,
        ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+1+offset,Caption);
    end;
    bm.Canvas.Font.Color := Font.Color;
    bm.Canvas.TextOut(((bm.Width-bm.Canvas.TextWidth(Caption)) div 2)+offset,
      ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,Caption);
    bm.Canvas.Brush.Style := bsSolid;
end;

procedure TGradBtn.DoInset(offset: Integer);
begin
  bm.Canvas.Brush.Style := bsClear;
  if FShadeType = stHeavy then begin
    bm.Canvas.Font.Color := FTxtShadowClr;
    bm.Canvas.TextOut(((bm.Width-bm.Canvas.TextWidth(Caption)) div 2)-1+offset,
      ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)-1+offset,Caption);
  end;
  bm.Canvas.Font.Color := FTxtHiliteClr;
  bm.Canvas.TextOut(((bm.Width-bm.Canvas.TextWidth(Caption)) div 2)+1+offset,
    ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+1+offset,Caption);
  bm.Canvas.Font.Color := Font.Color;
  bm.Canvas.TextOut(((bm.Width-bm.Canvas.TextWidth(Caption)) div 2)+offset,
    ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,Caption);
  bm.Canvas.Brush.Style := bsSolid;
end;

procedure TGradBtn.DoNorm(offset: Integer);
begin    //If no 3D effect is set, we just draw text in the center
    bm.Canvas.Brush.Style := bsClear;
    bm.Canvas.Font.Color := Font.Color;
    bm.Canvas.TextOut(((bm.Width-bm.Canvas.TextWidth(Caption)) div 2)+offset,
      ((bm.Height-bm.Canvas.TextHeight(Caption)) div 2)+offset,Caption);
    bm.Canvas.Brush.Style := bsSolid;
end;

destructor TGradBtn.Destroy;
begin
  bm.Free;       //Cleanup the bitmap - no memory leaks
  inherited Destroy;
end;

{I added the following to allow the Enter key to behave as a Click.}
 {If return key was pressed, do a 'Click'.  This only happens if
  we have the 'Focus'.}
procedure TGradBtn.KeyPress(var Key: Char);
begin
  inherited;
  if Key = Chr(13) then Click;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TGradBtn]);
end;

end.
